# Load R scripts containing the functions needed to run 1b_fetch_ancillary.
source("src/1_ancillary_download.R")

p1_ancillary_targets <- list(

  # Download ancillary data for sites that still appear in the cleaned up metals data (not
  # using sites from the raw metals data downloaded in `1a_fetch_metals.R` because that
  # would be way more data than we need to download). Go follow instructions in the
  # `metals-data-cleanup` codebase before running these targets.

  # Identify the filepath of cleaned metals data from the other repo
  tar_target(p1_sites_clean_csv, {
    site_info_clean_csv <- "../metals-data-cleanup/out/metals_clean_sites.csv"
    if(!file.exists(site_info_clean_csv)) {
      stop("As of 7/19/2022, there is a difference of ~ 10k sites between the raw metals data ",
           "and the cleaned metals data. As such, we are only going to download ancillary data ",
           "for those sites that appear in the cleaned data. You need to first run the `metals-data-cleanup` ",
           "companion pipeline so that the `metals-data-cleanup/out/metals_clean_sites.csv` ",
           "file exists before you can complete this ancillary data download.")
    } else {
      return(site_info_clean_csv)
    }
  }, format = 'file', error = "stop"),

  # Extract the site identifiers from the cleaned metals data
  tar_target(p1_sites_clean, read_csv(p1_sites_clean_csv) %>% pull(MonitoringLocationIdentifier)),

  # Generate a new site metadata table with the unique sites from
  # the cleaned metals data and their associated basin abbreviation
  tar_target(p1_site_basin_unique, unique_site_info_and_basin(
    wqp_site_info = p1_site_info_all_huc_metal_csv,
    basin_xwalk = basin_xwalk,
    sites_clean = p1_sites_clean)),

  # Group the sites into chunks of 500 within each 8-digit HUC. This doesn't download
  # the data, but sets up the download step to operate by these groups.
  tar_target(p1_site_basin_group, p1_site_basin_unique %>%
               add_download_groups(max_sites_allowed = 500) %>%
               group_by(download_grp) %>%
               tar_group(),
             iteration = "group"),

  # List the ancillary parameters to be downloaded. Add a column with a
  # filename-friendly version of the characteristic name to be used
  # when saving the file.
  tar_target(characteristicName, tibble(characteristicName = c(
    "Alkalinity",
    "Alkalinity, bicarbonate",
    "Alkalinity, carbonate",
    "Alkalinity, Phenolphthalein (total hydroxide+1/2 carbonate)",
    "Alkalinity, total",
    "Dissolved oxygen (DO)",
    "Dissolved oxygen saturation",
    "pH",
    "Conductivity",
    "Salinity",
    "Total dissolved solids",
    "Fixed suspended solids",
    "Total fixed solids",
    "Total solids",
    "Total suspended solids",
    "Total volatile solids",
    "Temperature, water",
    "Turbidity"),
    file_name = c(
      "Alkalinity",
      "Alkalinity_bicarbonate",
      "Alkalinity_carbonate",
      "Alkalinity_Phenolphthalein",
      "Alkalinity_total",
      "DO",
      "DO_saturation",
      "pH",
      "Conductivity",
      "Salinity",
      "TDS",
      "FSS",
      "TFS",
      "TS",
      "TSS",
      "TVS",
      "Temperature",
      "Turbidity"))),

  # For each download group defined earlier, download the ancillary
  # data per characteristic and save in individual CSV files per group.
  tar_target(p1_ancillary_data_csv,{
    group_basin <- unique(p1_site_basin_group$download_grp)
    download_ancillary_data_unique_site(
      siteid = p1_site_basin_group$MonitoringLocationIdentifier,
      characteristicName = characteristicName$characteristicName,
      sampleMedia = sampleMedia,
      file_out = sprintf("tmp_ancillary/wqp_data_%s_%s.csv", group_basin, characteristicName$file_name))
  }, pattern = cross(characteristicName, p1_site_basin_group), format = "file"),

  # Group all the ancillary data downloaded in the previous step into one CSV file
  tar_target(
    p1_all_ancillary_data_csv, {
      file_out <- 'out/all_ancillary_data.csv'
      p1_ancillary_data_csv %>%
        purrr::map(read_csv, col_types=get_wqp_col_types()) %>%
        bind_rows() %>%
        write_csv(file_out)
      return(file_out)
    },
    format = "file"
  )

)
