#' @title Grab the unique site locations from the water quality portal pull and
#' the basin xwalk
#' @description This function joins the site info and basin names.
#' @param wqp_site_info filepath for the csv that contains the site info from the wqp pull.
#' @param basin_xwalk list that matches the huc 4 with the water basin name.
#' @param sites_clean vector of site identifiers that exist in the final clean metals data
#' from the companion pipeline `metals-data-cleanup`

unique_site_info_and_basin<- function(wqp_site_info, basin_xwalk, sites_clean){
  site_info <- read_csv(wqp_site_info)%>%
    select(MonitoringLocationIdentifier, HUCEightDigitCode)%>%
    mutate(HUCFourDigitCode = str_sub(HUCEightDigitCode, 1, 4))%>%
    left_join(basin_xwalk, by = c("HUCFourDigitCode" = "hucs"))%>%
    filter(MonitoringLocationIdentifier %in% sites_clean)
  return(site_info)
}

#' @title Download water quality portal site data and information
#' @description Use this to download metal files from the water quality portal.
#' It has one function for  downloading the data and another
#' for site info. It returns a csv the unique combination of hucs and metals.
#' @param siteid character string of unique sites from metals data pull
#' @param characteristicName character string of the parameters of interest
#' @param sampleMedia character string for the search query to only include water
#' @param file_out csv filepath

download_ancillary_data_unique_site <- function(siteid, characteristicName, sampleMedia, file_out) {

  data_out<- retry::retry(readWQPdata(
    siteid = siteid,
    characteristicName = characteristicName,
    sampleMedia = sampleMedia),
    when = "Error:", max_tries = 3)

  write_csv(data_out, file = file_out)

  return(file_out)

}

#' @title Group sites for downloading without hitting the WQP cap
#' @description This function returns a data frame with a single column used
#' to group the sites into reasonably sized chunks for downloading data.
#' @param site_basin_info data.frame returned by `unique_site_info_and_basin()`
#' @param max_sites_allowed single, numeric value indicating the maximum number
#' of sites allowed in each download group.
#' @return a data.frame that matches `site_basin_info` but has one additional
#' column called `download_grp` which is made up of unique groups to `group_by()`
#' and then `tar_group()` for downloading.

add_download_groups <- function(site_basin_info, max_sites_allowed) {

  site_basin_info %>%
    # For each HUC, add a column that numbers the sites
    group_by(HUCEightDigitCode) %>%
    mutate(site_n = row_number()) %>%
    ungroup() %>%
    # Add a column indicating whether a site is over the `max_sites_allowed`
    mutate(site_grp = ifelse(site_n > max_sites_allowed, 2, 1)) %>%
    # Create a new column that creates more than one unique group per
    # HUC if there are more than `max_sites_allowed` in that HUC
    mutate(download_grp = sprintf("%s-%s", HUCEightDigitCode, site_grp)) %>%
    select(-site_n, -site_grp)
}

