#' @title Summarize the water quality portal pull to see what data was
#' added and how much more.
#'
#' @description This function joins the site info and metals wqp datasets.
#' @param wqp_metals_data filepath for the csv that contains the metals data from the wqp pull.
#' @param wqp_site_info filepath for the csv that contains the site info from the wqp pull.
#' @param basin_xwalk list that matches the huc 4 with the water basin name.


join_and_prep_metadata<- function(wqp_metals_data, wqp_site_info, basin_xwalk){
  site_info <- read_csv(wqp_site_info)%>%
    select(MonitoringLocationIdentifier, MonitoringLocationTypeName,
           HUCEightDigitCode, LatitudeMeasure, LongitudeMeasure)
  metals_data <- read_csv(wqp_metals_data)

  metals_site_joined_data<- metals_data%>%
    select(MonitoringLocationIdentifier, ActivityStartDate, CharacteristicName,
           ResultMeasureValue)%>%
    left_join(site_info, by = "MonitoringLocationIdentifier")%>%
    mutate(HUCFourDigitCode = str_sub(HUCEightDigitCode, 1, 4))%>%
    left_join(basin_xwalk, by = c("HUCFourDigitCode" = "hucs"))
  return(metals_site_joined_data)
}

#' @description Function that summarizes the wqp pull changes grouped by
#' basin. It returns a .csv with a table that summarizes the changes.
#' @param all_wqp_data_info merged site info and metals data that
#' has been reduced to only include appropriate variables for summarizing.
#' @param file_out csv filepath
summarize_wqp_metals <- function(file_out, all_wqp_data_info){

  metals_data_summary<- all_wqp_data_info%>%
    group_by(basin)%>%
    summarize(obs = n(),
              num_na = sum(is.na(ResultMeasureValue)),
              unique_sites = length(unique(MonitoringLocationIdentifier)),
              first_date = min(ActivityStartDate),
              last_date = max(ActivityStartDate))%>%
    write_csv(file_out)
  return(file_out)
}
#' @description Function that summarizes the changes from a wqp pull organized
#' by metal and basin. The function returns a csv with a table that summarizes
#' these changes.
#' @param all_wqp_data_info merged site info and metals data that
#' has been reduced to only include appropriate variables for summarizing.
#' @param file_out csv filepath

summarize_basins_by_metals <- function(file_out, all_wqp_data_info){
    by_metal_basin_data_summary<- all_wqp_data_info%>%
      group_by(CharacteristicName, basin)%>%
      summarize(obs = n())%>%
      pivot_wider(id_cols = basin, names_from = CharacteristicName, values_from = obs, values_fill = list(obs = 0))%>%
    write_csv(file_out)
  return(file_out)
}






